from integration_tests.helpers.http_methods_helpers import get, post
import pytest


@pytest.mark.benchmark
def test_get_sync_view(session):
    r = get("/sync/view")
    assert r.text == "Hello, world!"


@pytest.mark.benchmark
def test_post_sync_view(session):
    r = post("/sync/view", data={"name": "John"})
    assert "John" in r.text


@pytest.mark.benchmark
def test_get_sync_decorator_view(session):
    r = get("/sync/view/decorator")
    assert r.text == "Hello, world!"


@pytest.mark.benchmark
def test_post_sync_decorator_view(session):
    r = post("/sync/view/decorator", data={"name": "John"})
    assert "John" in r.text


@pytest.mark.benchmark
def test_get_async_view(session):
    r = get("/async/view")
    assert r.text == "Hello, world!"


@pytest.mark.benchmark
def test_post_async_view(session):
    r = post("/async/view", data={"name": "John"})
    assert "John" in r.text


@pytest.mark.benchmark
def test_get_async_decorator_view(session):
    r = get("/async/view/decorator")
    assert r.text == "Hello, world!"


@pytest.mark.benchmark
def test_post_async_decorator_view(session):
    r = post("/async/view/decorator", data={"name": "John"})
    assert "John" in r.text



import pytest
from integration_tests.helpers.http_methods_helpers import post


@pytest.mark.benchmark
@pytest.mark.parametrize("function_type", ["sync", "async"])
def test_post(function_type: str, session):
    res = post(f"/{function_type}/dict")
    assert res.text == f"{function_type} dict post"
    assert function_type in res.headers
    assert res.headers[function_type] == "dict"


@pytest.mark.benchmark
@pytest.mark.parametrize("function_type", ["sync", "async"])
def test_post_with_param(function_type: str, session):
    res = post(f"/{function_type}/body", data={"hello": "world"})
    assert res.text == "hello=world"



import pytest

from integration_tests.helpers.http_methods_helpers import get


@pytest.mark.benchmark
def test_global_dependency_injection(benchmark):
    r = get("/sync/global_di")
    assert r.status_code == 200
    assert r.text == "GLOBAL DEPENDENCY"


@pytest.mark.benchmark
def test_router_dependency_injection(benchmark):
    r = get("/sync/router_di")
    assert r.status_code == 200
    assert r.text == "ROUTER DEPENDENCY"


@pytest.mark.benchmark
def test_subrouter_global_dependency_injection(benchmark):
    r = get("/di_subrouter/subrouter_global_di")
    assert r.status_code == 200
    assert r.text == "GLOBAL DEPENDENCY"


@pytest.mark.benchmark
def test_subrouter_router_dependency_injection(benchmark):
    r = get("/di_subrouter/subrouter_router_di")
    assert r.status_code == 200
    assert r.text == "ROUTER DEPENDENCY"



"""This is Robyn's dependency injection file."""

from typing import Any


class DependencyMap:
    def __init__(self):
        self.global_dependency_map: dict[str, Any] = {}
        # {'router': {'dependency_name': dependency_class}
        self.router_dependency_map: dict[str, dict[str, Any]] = {}

    def add_router_dependency(self, router, **kwargs):
        """Adds a dependency to a route.

        Args:
            router App Object: The route to add the dependency to.
            kwargs (dict): The dependencies to add to the route.
        """
        if router not in self.router_dependency_map:
            self.router_dependency_map[router] = {}

        self.router_dependency_map[router].update(**kwargs)

    def add_global_dependency(self, **kwargs):
        """Adds a dependency to all routes.

        Args:
            kwargs (dict): The dependencies to add to all routes.
        """
        for name, element in kwargs.items():
            self.global_dependency_map.update({name: element})

    def get_router_dependencies(self, router):
        """Gets the dependencies for a specific route.

        Args:
            router

        Returns:
            dict: The dependencies for the specified route.
        """
        return self.router_dependency_map.get(router, {})

    def get_global_dependencies(self):
        """Gets the dependencies for a route.

        Args:
            route (str): The route to get the dependencies for.
        """
        return self.global_dependency_map

    def merge_dependencies(self, target_router):
        """
        Merge dependencies from this DependencyMap into another router's DependencyMap.

        Args:
            target_router: The router with which to merge dependencies.

        This method iterates through the dependencies of this DependencyMap and adds any dependencies
        that are not already present in the target router's DependencyMap.
        """
        for dep_key in self.get_global_dependencies():
            if dep_key in target_router.dependencies.get_global_dependencies():
                continue
            target_router.dependencies.get_global_dependencies()[dep_key] = self.get_global_dependencies()[dep_key]

    def get_dependency_map(self, router) -> dict:
        return {
            "global_dependencies": self.get_global_dependencies(),
            "router_dependencies": self.get_router_dependencies(router),
        }



from collections.abc import Callable
import pytest
from integration_tests.helpers.http_methods_helpers import get, post, put


@pytest.mark.benchmark
@pytest.mark.usefixtures("session")
@pytest.mark.parametrize(
    "route,method",
    [
        ("/sync/get/no_dec", get),
        ("/async/get/no_dec", get),
        ("/sync/put/no_dec", put),
        ("/async/put/no_dec", put),
        ("/sync/post/no_dec", post),
        ("/async/post/no_dec", post),
    ],
)
def test_exception_handling(route: str, method: Callable):
    r = method(route, expected_status_code=200)
    assert r.text == "Success!"



import json

from websocket import create_connection
import pytest

BASE_URL = "ws://127.0.0.1:8080"


@pytest.mark.benchmark
def test_web_socket_raw_benchmark(session):
    ws = create_connection(f"{BASE_URL}/web_socket")
    assert ws.recv() == "Hello world, from ws"

    ws.send("My name is?")
    assert ws.recv() == "This is a broadcast message"
    assert ws.recv() == "This is a message to self"
    assert ws.recv() == "Whaaat??"

    ws.send("My name is?")
    assert ws.recv() == "Whooo??"

    ws.send("My name is?")
    assert ws.recv() == "*chika* *chika* Slim Shady."


def test_web_socket_json(session):
    """
    Not using this as the benchmark test since this involves JSON marshalling/unmarshalling
    which pollutes the benchmark measurement.
    """
    ws = create_connection(f"{BASE_URL}/web_socket_json")
    assert ws.recv() == "Hello world, from ws"

    msg = "My name is?"

    ws.send(msg)
    resp = json.loads(ws.recv())
    assert resp["resp"] == "Whaaat??"
    assert resp["msg"] == msg

    ws.send(msg)
    resp = json.loads(ws.recv())
    assert resp["resp"] == "Whooo??"
    assert resp["msg"] == msg

    ws.send(msg)
    resp = json.loads(ws.recv())
    assert resp["resp"] == "*chika* *chika* Slim Shady."
    assert resp["msg"] == msg


def test_websocket_di(session):
    """
    Not using this as the benchmark test since this involves JSON marshalling/unmarshalling

    """

    msg = "GLOBAL DEPENDENCY ROUTER DEPENDENCY"

    ws = create_connection(f"{BASE_URL}/web_socket_di")
    assert ws.recv() == msg
